package com.sun.corba.se.impl.activation;

import com.sun.corba.se.spi.activation.InvalidORBid;
import java.util.Iterator;
import java.util.NoSuchElementException;
import com.sun.corba.se.spi.activation.ServerHeldDown;
import com.sun.corba.se.spi.activation.ORBPortInfo;
import com.sun.corba.se.spi.activation.ORBAlreadyRegistered;
import com.sun.corba.se.spi.activation.EndPointInfo;
import org.omg.CORBA.SystemException;
import com.sun.corba.se.impl.logging.ActivationSystemException;
import com.sun.corba.se.spi.activation.RepositoryPackage.ServerDef;
import com.sun.corba.se.spi.activation.Server;
import java.util.HashMap;

public class ServerTableEntry
{
    private static final int DE_ACTIVATED = 0;
    private static final int ACTIVATING = 1;
    private static final int ACTIVATED = 2;
    private static final int RUNNING = 3;
    private static final int HELD_DOWN = 4;
    private static final long waitTime = 2000L;
    private static final int ActivationRetryMax = 5;
    private int state;
    private int serverId;
    private HashMap orbAndPortInfo;
    private Server serverObj;
    private ServerDef serverDef;
    private Process process;
    private int activateRetryCount;
    private String activationCmd;
    private ActivationSystemException wrapper;
    private static String javaHome;
    private static String classPath;
    private static String fileSep;
    private static String pathSep;
    private boolean debug;
    
    private String printState() {
        String s = "UNKNOWN";
        switch (this.state) {
            case 0: {
                s = "DE_ACTIVATED";
                break;
            }
            case 1: {
                s = "ACTIVATING  ";
                break;
            }
            case 2: {
                s = "ACTIVATED   ";
                break;
            }
            case 3: {
                s = "RUNNING     ";
                break;
            }
            case 4: {
                s = "HELD_DOWN   ";
                break;
            }
        }
        return s;
    }
    
    @Override
    public String toString() {
        return "ServerTableEntry[state=" + this.printState() + " serverId=" + this.serverId + " activateRetryCount=" + this.activateRetryCount + "]";
    }
    
    ServerTableEntry(ActivationSystemException wrapper, int n, ServerDef serverDef, int i, String str, boolean b, boolean debug) {
        this.activateRetryCount = 0;
        this.debug = false;
        this.wrapper = wrapper;
        this.serverId = n;
        this.serverDef = serverDef;
        this.debug = debug;
        this.orbAndPortInfo = new HashMap(255);
        this.activateRetryCount = 0;
        this.state = 1;
        this.activationCmd = ServerTableEntry.javaHome + ServerTableEntry.fileSep + "bin" + ServerTableEntry.fileSep + "java " + serverDef.serverVmArgs + " -Dioser=" + System.getProperty("ioser") + " -D" + "org.omg.CORBA.ORBInitialPort" + "=" + i + " -D" + "com.sun.CORBA.activation.DbDir" + "=" + str + " -D" + "com.sun.CORBA.POA.ORBActivated" + "=true -D" + "com.sun.CORBA.POA.ORBServerId" + "=" + n + " -D" + "com.sun.CORBA.POA.ORBServerName" + "=" + serverDef.serverName + " " + (b ? "-Dcom.sun.CORBA.activation.ORBServerVerify=true " : "") + "-classpath " + ServerTableEntry.classPath + (serverDef.serverClassPath.equals("") ? "" : ServerTableEntry.pathSep) + serverDef.serverClassPath + " com.sun.corba.se.impl.activation.ServerMain " + serverDef.serverArgs + (debug ? " -debug" : "");
        if (debug) {
            System.out.println("ServerTableEntry constructed with activation command " + this.activationCmd);
        }
    }
    
    public int verify() {
        try {
            if (this.debug) {
                System.out.println("Server being verified w/" + this.activationCmd);
            }
            this.process = Runtime.getRuntime().exec(this.activationCmd);
            int wait = this.process.waitFor();
            if (this.debug) {
                this.printDebug("verify", "returns " + ServerMain.printResult(wait));
            }
            return wait;
        }
        catch (Exception obj) {
            if (this.debug) {
                this.printDebug("verify", "returns unknown error because of exception " + obj);
            }
            return 4;
        }
    }
    
    private void printDebug(String str, String str2) {
        System.out.println("ServerTableEntry: method  =" + str);
        System.out.println("ServerTableEntry: server  =" + this.serverId);
        System.out.println("ServerTableEntry: state   =" + this.printState());
        System.out.println("ServerTableEntry: message =" + str2);
        System.out.println();
    }
    
    synchronized void activate() throws SystemException {
        this.state = 2;
        try {
            if (this.debug) {
                this.printDebug("activate", "activating server");
            }
            this.process = Runtime.getRuntime().exec(this.activationCmd);
        }
        catch (Exception ex) {
            this.deActivate();
            if (this.debug) {
                this.printDebug("activate", "throwing premature process exit");
            }
            throw this.wrapper.unableToStartProcess();
        }
    }
    
    synchronized void register(Server serverObj) {
        if (this.state == 2) {
            this.serverObj = serverObj;
            if (this.debug) {
                this.printDebug("register", "process registered back");
            }
            return;
        }
        else {
            if (this.debug) {
                this.printDebug("register", "throwing premature process exit");
            }
            throw this.wrapper.serverNotExpectedToRegister();
        }
    }
    
    synchronized void registerPorts(String s, EndPointInfo[] array) throws ORBAlreadyRegistered {
        if (this.orbAndPortInfo.containsKey(s)) {
            throw new ORBAlreadyRegistered(s);
        }
        int length = array.length;
        EndPointInfo[] value = new EndPointInfo[length];
        for (int i = 0; i < length; ++i) {
            value[i] = new EndPointInfo(array[i].endpointType, array[i].port);
            if (this.debug) {
                System.out.println("registering type: " + value[i].endpointType + "  port  " + value[i].port);
            }
        }
        this.orbAndPortInfo.put(s, value);
        if (this.state == 2) {
            this.state = 3;
            this.notifyAll();
        }
        if (this.debug) {
            this.printDebug("registerPorts", "process registered Ports");
        }
        return;
    }
    
    void install() {
        Server serverObj = null;
        synchronized (this) {
            if (this.state == 3) {
                serverObj = this.serverObj;
            }
            else {
                throw this.wrapper.serverNotRunning();
            }
        }
        if (serverObj != null) {
            serverObj.install();
        }
    }
    
    void uninstall() {
        Server serverObj = null;
        Process process = null;
        synchronized (this) {
            serverObj = this.serverObj;
            process = this.process;
            if (this.state == 3) {
                this.deActivate();
            }
            else {
                throw this.wrapper.serverNotRunning();
            }
        }
        try {
            if (serverObj != null) {
                serverObj.shutdown();
                serverObj.uninstall();
            }
            if (process != null) {
                process.destroy();
            }
        }
        catch (Exception ex) {}
    }
    
    synchronized void holdDown() {
        this.state = 4;
        if (this.debug) {
            this.printDebug("holdDown", "server held down");
        }
        this.notifyAll();
    }
    
    synchronized void deActivate() {
        this.state = 0;
        if (this.debug) {
            this.printDebug("deActivate", "server deactivated");
        }
        this.notifyAll();
    }
    
    synchronized void checkProcessHealth() {
        if (this.state == 3) {
            try {
                this.process.exitValue();
            }
            catch (IllegalThreadStateException ex) {
                return;
            }
            synchronized (this) {
                this.orbAndPortInfo.clear();
                this.deActivate();
            }
        }
    }
    
    synchronized boolean isValid() {
        if (this.state == 1 || this.state == 4) {
            if (this.debug) {
                this.printDebug("isValid", "returns true");
            }
            return true;
        }
        else {
            try {
                this.process.exitValue();
            }
            catch (IllegalThreadStateException ex) {
                return true;
            }
            if (this.state == 2) {
                if (this.activateRetryCount < 5) {
                    if (this.debug) {
                        this.printDebug("isValid", "reactivating server");
                    }
                    ++this.activateRetryCount;
                    this.activate();
                    return true;
                }
                else {
                    if (this.debug) {
                        this.printDebug("isValid", "holding server down");
                    }
                    this.holdDown();
                    return true;
                }
            }
            else {
                this.deActivate();
                return false;
            }
        }
    }
    
    synchronized ORBPortInfo[] lookup(String anObject) throws ServerHeldDown {
        while (true) {
            if (this.state != 1 && this.state != 2) {
                break;
            }
            try {
                this.wait(2000L);
                if (!this.isValid()) {
                    break;
                }
                continue;
            }
            catch (Exception ex) {}
        }
        if (this.state == 3) {
            ORBPortInfo[] array = new ORBPortInfo[this.orbAndPortInfo.size()];
            Iterator<Object> iterator = this.orbAndPortInfo.keySet().iterator();
            try {
                String key;
                int port;
                for (int n = 0; iterator.hasNext(); array[n] = new ORBPortInfo(key, port), ++n) {
                    key = (String)iterator.next();
                    EndPointInfo[] array2 = (EndPointInfo[])(EndPointInfo[])this.orbAndPortInfo.get(key);
                    port = -1;
                    int i = 0;
                    while (i < array2.length) {
                        if (this.debug) {
                            System.out.println("lookup num-ports " + array2.length + "   " + array2[i].endpointType + "   " + array2[i].port);
                        }
                        if (array2[i].endpointType.equals(anObject)) {
                            port = array2[i].port;
                            break;
                        }
                        ++i;
                    }
                }
            }
            catch (NoSuchElementException ex2) {}
            return array;
        }
        if (this.debug) {
            this.printDebug("lookup", "throwing server held down error");
        }
        throw new ServerHeldDown(this.serverId);
    }
    
    synchronized EndPointInfo[] lookupForORB(String key) throws ServerHeldDown, InvalidORBid {
        while (true) {
            if (this.state != 1 && this.state != 2) {
                break;
            }
            try {
                this.wait(2000L);
                if (!this.isValid()) {
                    break;
                }
                continue;
            }
            catch (Exception ex) {}
        }
        if (this.state == 3) {
            EndPointInfo[] array2;
            try {
                EndPointInfo[] array = (EndPointInfo[])(EndPointInfo[])this.orbAndPortInfo.get(key);
                array2 = new EndPointInfo[array.length];
                for (int i = 0; i < array.length; array2[i] = new EndPointInfo(array[i].endpointType, array[i].port), ++i) {
                    if (this.debug) {
                        System.out.println("lookup num-ports " + array.length + "   " + array[i].endpointType + "   " + array[i].port);
                    }
                }
            }
            catch (NoSuchElementException ex2) {
                throw new InvalidORBid();
            }
            return array2;
        }
        if (this.debug) {
            this.printDebug("lookup", "throwing server held down error");
        }
        throw new ServerHeldDown(this.serverId);
    }
    
    synchronized String[] getORBList() {
        String[] array = new String[this.orbAndPortInfo.size()];
        Iterator<Object> iterator = this.orbAndPortInfo.keySet().iterator();
        try {
            int n = 0;
            while (iterator.hasNext()) {
                array[n++] = (String)iterator.next();
            }
        }
        catch (NoSuchElementException ex) {}
        return array;
    }
    
    int getServerId() {
        return this.serverId;
    }
    
    boolean isActive() {
        return this.state == 3 || this.state == 2;
    }
    
    synchronized void destroy() {
        Server serverObj = null;
        Process process = null;
        synchronized (this) {
            serverObj = this.serverObj;
            process = this.process;
            this.deActivate();
        }
        try {
            if (serverObj != null) {
                serverObj.shutdown();
            }
            if (this.debug) {
                this.printDebug("destroy", "server shutdown successfully");
            }
        }
        catch (Exception obj) {
            if (this.debug) {
                this.printDebug("destroy", "server shutdown threw exception" + obj);
            }
        }
        try {
            if (process != null) {
                process.destroy();
            }
            if (this.debug) {
                this.printDebug("destroy", "process destroyed successfully");
            }
        }
        catch (Exception obj2) {
            if (this.debug) {
                this.printDebug("destroy", "process destroy threw exception" + obj2);
            }
        }
    }
    
    static {
        ServerTableEntry.javaHome = System.getProperty("java.home");
        ServerTableEntry.classPath = System.getProperty("java.class.path");
        ServerTableEntry.fileSep = System.getProperty("file.separator");
        ServerTableEntry.pathSep = System.getProperty("path.separator");
    }
}
